require 'helpers'
--vars--
local screenWidth, screenHeight = love.graphics.getDimensions()
local shapeColor = { 180, 200, 220, 255 }
local gravity = 3.7
local terminalVelocity = 8
-- -- --

local function isScreenLimit(object)
  local newY = object.y + object.vSpeed

  if newY + object.height > screenHeight then
    return true
  end
  return false
end

local function move(self)
  if isScreenLimit(self) then
    print("true")
    self.y = screenHeight - self.height
    self.x = self.x + self.hSpeed
    self.vSpeed = -(self.vSpeed / self.bounce)
    self.isLanded = true
    self.isThrustingUp = false
  else
    self.isLanded = false
    self.y = self.y + self.vSpeed
    self.x = self.x + self.hSpeed
  end
end

local function drawTrajectory(self, num)
  local currX = self.x + self.width / 2
  local currY = self.y + self.height / 2
  local currVSpeed = self.vSpeed

  for i = 1, num do
    currVSpeed = currVSpeed < terminalVelocity and currVSpeed + gravity * love.timer.getAverageDelta() or terminalVelocity
    currX = currX + self.hSpeed
    currY = currY + currVSpeed

    love.graphics.setColor(230, 40, 0, 255)
    love.graphics.circle("fill", currX, currY, 5)
  end
end

local function updateShape(self, dt)
  --update speed
  if not self.isLanded then self.vSpeed = self.vSpeed < terminalVelocity and self.vSpeed + gravity * dt or terminalVelocity end
  --add thrusters
  if love.keyboard.isDown('w', 's', 'a', 'd', 'space') then
    if love.keyboard.isDown('w') then
      if self.isLanded then self.vSpeed = self.vSpeed - self.thrustPower * dt end
    end
    if love.keyboard.isDown('s') then self.vSpeed = self.vSpeed + self.thrustPower * dt end
    if love.keyboard.isDown('a') then self.hSpeed = self.hSpeed - self.sideThrustPower * dt end
    if love.keyboard.isDown('d') then self.hSpeed = self.hSpeed + self.sideThrustPower * dt end
    if love.keyboard.isDown('space') then self.isThrustingUp = true end
  end
  --update position
  if self.isThrustingUp then
    self:move()
  end
end

local function drawShape(self)
  love.graphics.setColor(shapeColor)
  love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
  self:drawTrajectory(3000)
end

local function makeNewShape(x, y, width, height)
  local newShape = helpers.makeRect(x - width / 2, y - height / 2, width, height)
  newShape.vSpeed = 0
  newShape.hSpeed = 0
  newShape.bounce = 6
  newShape.isLanded = false
  newShape.isThrustingUp = true
  newShape.sideThrustPower = 13
  newShape.thrustPower = 7
  newShape.drawTrajectory = drawTrajectory
  newShape.update = updateShape
  newShape.draw = drawShape
  newShape.move = move


  return newShape
end

--Methods
shape =
{
  makeNewShape = makeNewShape
}